import vobject
import parsing

class Component:
    """Contains a dictionary, keyed by name to list of values."""
    def __init__(self):
        self.dict = {}
    def name(self):
        return self.dict["BEGIN"][0]
    def value(self):
        return self.dict
    def add(self, cv):
        """Takes a Contval object, and adds its contents to this component."""
        addToDictList(self.dict, cv.name(), cv.value())
    def __str__(self):
        return "<Component %s|dict:%s>" % (str(self.name()), str(self.dict))
    def __repr__(self):
        return self.__str__()

class Contval:
    """Temporary objects for internal use; simply a value bound to a name."""
    def __init__(self, name, value):
        self.n = name
        self.v = value
    def name(self):
        return self.n
    def value(self):
        return self.v
    def __str__(self):
        return "<Contval %s|value:%s>" % (str(self.name()), str(self.value()))
    def __repr__(self):
        return self.__str__()

def addToDictList(dict, key, val):
    """This utility function adds to a special kind of dictionary,
    where the values are lists.
    """
    if dict.has_key(key):
        dict[key].append(val)
    else:
        dict[key] = [val]
        
def contentLinesToComponentsHelper (lst):
    def begin(x): return x.name.lower() == "begin"
    def end(x):   return x.name.lower() == "end"
    results = [] # Store things here.
    while True:
        if lst == []:
            # Degenerate case.
            return results, []
        nextItem = lst[0]
        if begin(nextItem):
            # New subcomponent found. Form that component and append it to result list.
            result,remainingList = contentLinesToComponentsHelper(lst[1:])
            comp = Component()
            map(lambda x: comp.add(x), [Contval(nextItem.name, parsing.parse(nextItem))] + result)
            results.append(comp)
            lst = remainingList
        elif end(nextItem):
            # We're done parsing a component, time to return all we've accumulated.
            results.append(Contval(nextItem.name, parsing.parse(nextItem)))
            return results, lst[1:]
        elif lst == []:
            # Error. Apparently we've hit end of file without having located a component's end marker.
            raise "nothing left"
        else:
            # Simply add to result list and prepare for next iteration.
            # The reason that "lst" is reduced here, instead of outside this if-statement,
            # is because you don't actually want it to be decremented after a BEGIN is found and
            # a subcomponent added. Due to the way the code up there works.
            lst = lst[1:]
            results.append(Contval(nextItem.name, parsing.parse(nextItem)))

def contentLinesToComponents (contentLines):
    return contentLinesToComponentsHelper(contentLines)[0]

def deUnicode(contentline):
    #name = contentline.name.encode() #let's not encode name
    value = contentline.value.encode()
    params = []
    for key in contentline.params:
        val = contentline.params[key]
        tmp = [key.encode()]
        tmp.extend(map(lambda x: x.encode(), val))
        params.append(tmp)
    newContentline = vobject.ContentLine(contentline.name, params, value)
    return newContentline

def readComponentsFromFile (fname):
    """Parse list of components from file, which this API's user might want to use."""
    f     = None
    lines = []
    try:
        f = open(fname, "r")
        for line in vobject.getLogicalLines(f):
            nextline = vobject.parseLine(line)
            if nextline == None:
                break
            lines.append(nextline)
    finally:
        f.close()
    contentLines = map(lambda x: vobject.ContentLine(x[0],x[1],x[2]), lines)
    components   = contentLinesToComponents(contentLines)
    return components



if __name__ == '__main__':
    # Test of reading components from sampledata.txt
    comps = readComponentsFromFile("sampledata.txt")
    print "sampledata->components:\n %s" % comps
    if not 22 == len(comps):
        raise "test failed"
