"""Datastructures for user access."""

from dateutil import rrule


# class Date:
#     def __init__(self, datetime, timezone):
#         self.datetime = datetime
#         self.timezone = timezone
#     def __str__(self):
#         return "<date|datetime:%s,timezone:%s>" % (str(self.datetime), str(self.timezone))
#     def __repr__(self):
#         return self.__str__()

class Period:
    def __init__(self, initDateTime):
        self.initDateTime = initDateTime

class PeriodExplicit (Period):
    def __init__(self, initDateTime, endDateTime):
        self.initDateTime = initDateTime
        self.endDateTime = endDateTime

class PeriodStart (Period):
    def __init__(self, initDateTime, duration):
        self.initDateTime = initDateTime
        self.duration = duration

class Text:
    def __init__(self, values, altRepresentation, language):
        self.values            = values
        self.altRepresentation = altRepresentation
        self.language          = language
    def __str__(self):
        return "<TEXT|altrep:%s,language:%s,values:%s>" % \
               (str(self.altRepresentation), str(self.language), str(self.values))
    def __repr__(self):
        return self.__str__()

class VCalendar:
    def __init__(self, prodid, version, components, calscale=None, method=None):
        self.prodid     = prodid
        self.version    = version
        self.calscale   = calscale
        self.method     = method
        self.components = components

class VTimeZone:
    def __init__(self, tzid, standardcs, daylightcs, lastMod=None, tzurl=None):
        self.tzid       = tzid
        self.lastMod    = lastMod
        self.tzurl      = tzurl 
        self.standardcs = standardcs
        self.daylightcs = daylightcs

class InternalTimeZoneComponent (VTimeZone):
    def __init__(self, dtstart, tzoffsetto, tzoffsetfrom, comment, rdate, rrule, tzname): #check which are optional
        self.dtstart       = dtstart
        self.tzoffsetto    = tzoffsetto
        self.tzoffsetfrom  = tzoffsetfrom 
        self.comment       = comment
        self.rdate         = rdate
        self.rrule         = rrule
        self.tzname        = tzname    

class DaylightSavingsTimeZone (VTimeZone):
    pass

class StandardTimeZone (VTimeZone):
    pass    


class Organizer:
    def __init__(self, calAddress, fields):
        self.calAddress = calAddress
        self.fields = fields
    def __str__(self):
        return "<ORGANIZER|calAddress:%s,fields:%s>" % (str(self.calAddress), str(self.fields))
    def __repr__(self):
        return self.__str__()

class FreeBusy:
    def __init__(self, periods, type):
        self.periods = periods
        self.type = type
    def __str__(self):
        return "<FREEBUSY|periods:%s,type:%s>" % (str(self.periods), str(self.type))
    def __repr__(self):
        return self.__str__()

class Attendee:
    def __init__(self, calAddress, fields):
        self.calAddress = calAddress
        self.fields = fields
    def __str__(self):
        return "<ATTENDEE|calAddress:%s,fields:%s>" % (str(self.calAddress), str(self.fields))
    def __repr__(self):
        return self.__str__()

class Rdate:
    def __init__(self):
        raise "Rdate should not be directly instanced."
    def __str__(self):
        return "<%s|values:%s,tzid:%s>" % ("RDATE", str(self.values), str(self.tzid))
    def __repr__(self):
        return self.__str__()

class DateTimeRdate (Rdate):
    def __init__(self, values, tzid):
        self.values = values
        self.tzid   = tzid
    def __str__(self):
        return "<%s|values:%s,tzid:%s>" % ("DATETIMERDATE", str(self.values), str(self.tzid))
    def __repr__(self):
        return self.__str__()

class DateRdate (Rdate):
    def __init__(self, values, tzid):
        self.values = values
        self.tzid   = tzid
    def __str__(self):
        return "<%s|values:%s,tzid:%s>" % ("DATERDATE", str(self.values), str(self.tzid))
    def __repr__(self):
        return self.__str__()

class PeriodRdate (Rdate):
    def __init__(self, values, tzid):
        self.values = values
        self.tzid   = tzid
    def __str__(self):
        return "<%s|values:%s,tzid:%s>" % ("PERIODRDATE", str(self.values), str(self.tzid))
    def __repr__(self):
        return self.__str__()

class Trigger:
    def __init__(self, duration, related):
        self.duration = duration
        self.related  = related
    def __str__(self):
        return "<%s|duration:%s,related:%s>" % ("TRIGGER", str(self.duration), str(self.related))
    def __repr__(self):
        return self.__str__()

class Attachment:
    def __init__(self, format, url):
        self.format = format
        self.url    = url

class GeoPosition:
    def __init__(self, latitude, longitude):
        self.latitude  = latitude
        self.longitude = longitude
    def __str__(self):
        return "<%s|latitude:%s,longitude:%s>" % ("GEOPOSITION", str(self.latitude), str(self.longitude))
    def __repr__(self):
        return self.__str__()

class Duration:
    def __init__(self, sign, timedelta):
        self.sign      = sign
        self.timedelta = timedelta
    def __str__(self):
        return "<%s|sign:%s,timedelta:%s>" % ("DURATION", str(self.sign), str(self.timedelta))
    def __repr__(self):
        return self.__str__()



## Recurrence rule

# Constants required for dateutil.rrule.rrule()
WEEKDAYS = {"MO":0, "TU":1, "WE":2, "TH":3, "FR":4, "SA":5, "SU":6}
WEEKDAYS_FN = {"MO":rrule.MO, "TU":rrule.TU, "WE":rrule.WE,
               "TH":rrule.TH, "FR":rrule.FR, "SA":rrule.SA,
               "SU":rrule.SU}
POSSIBLE_LISTS = ["bysecond", "byminute", "byhour", "byday",
                  "bymonthday", "byyearday", "byweekno", "bymonth",
                  "bysetpos"]
FREQUENCIES = {"SECONDLY":rrule.SECONDLY,  "MINUTELY":rrule.MINUTELY,
               "HOURLY":  rrule.HOURLY,    "DAILY"   :rrule.DAILY,
               "WEEKLY":  rrule.WEEKLY,    "MONTHLY" :rrule.MONTHLY,
               "YEARLY":  rrule.YEARLY}

def stringToWeekDay (s):
    numText = ""
    dayText = ""
    for i in s:
        if i in string.letters:
            dayText = dayText + i
        else:
            numText = numText + i
    if numText == "":
        return WEEKDAYS[dayText]
    else:
        num = int(numText)
        val = (WEEKDAYS_FN[dayText](num))
        return val

def stringToRecurrence(s): #TODO: need to convert values into datetime, like with "until" below
    fields = string.split(s,";")
    table = {}
    for i in fields:
        key,val = string.split(i,"=",2)

        #RRULE renamed "byday"
        if key.lower() == "byday":
            key = "byweekday"
        #convert names to constants desired by RRULE
        if key.lower() == "byweekday":
            lst = []
            for v in string.split(val,","):
                lst.append( stringToWeekDay(v) )
            val = lst
        #convert names to constants defined in RRULE
        if key.lower() == "freq":
            val = FREQUENCIES[val]
        if key.lower() == "until":
            val = stringToDateTime(val)
        #these may be integers or (what we care about here) lists of integers
        if key.lower() in POSSIBLE_LISTS:
            if (type(val) == type("")) and ("," in val):
                val = map(lambda x: int(x), string.split(val,","))
        #try converting val into an integer and see what happens
        try:
            if type(val) == type(""):
                val = int(val)
        except:
            pass

        table[key.lower()] = val
    freq = table.pop("freq")
    return rrule.rrule(freq, **table)

#print stringToRecurrence("FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1")
#print "xrecur: " + str(list(stringToRecurrence("FREQ=YEARLY;INTERVAL=2;BYMONTH=1;BYDAY=SU;BYHOUR=8,9;BYMINUTE=30")))
#print "xrecur: " + str(list(stringToRecurrence("FREQ=DAILY;COUNT=10;INTERVAL=2")))
#print "xrecur: " + str(stringToRecurrence("FREQ=YEARLY;INTERVAL=2;BYMONTH=1;BYDAY=SU;BYHOUR=8,9;BYMINUTE=30") [0:4])


class Recurrence:
    def __init__(self, string):
        self.string = string
    def rrule(self):
        return rrule.rrulestr(self.string)
    def __str__(self):
        return "<%s|string:%s>" % ("recurrence", str(self.string))
    def __repr__(self):
        return self.__str__()
