"""Long or boring tests for vobjects."""

import vobject, behavior, registry

#------------------- Testing and running functions -----------------------------
def _test():
    import doctest, vobject, tests, icalendar, __init__
    doctest.testmod(vobject, verbose=0)
    doctest.testmod(tests, verbose=0)
    doctest.testmod(icalendar, verbose=0)
    doctest.testmod(__init__, verbose=0)
    
if __name__ == '__main__':
    _test()
    
#------------------------------ Tests ------------------------------------------
# See http://cvs.sourceforge.net/viewcvs.py/freeassociation/libical/test-data/
# for good sampledata.


testSilly="""
sillyname:name
profile:sillyprofile
stuff:folded
 line
""" + "morestuff;asinine:this line is not folded, \
but in practice probably ought to be, as it is exceptionally long, \
and moreover demonstratively stupid"

icaltest="""BEGIN:VCALENDAR
CALSCALE:GREGORIAN
X-WR-TIMEZONE;VALUE=TEXT:US/Pacific
METHOD:PUBLISH
PRODID:-//Apple Computer\, Inc//iCal 1.0//EN
X-WR-CALNAME;VALUE=TEXT:Example
VERSION:2.0
BEGIN:VEVENT
SEQUENCE:5
DTSTART;TZID=US/Pacific:20021028T140000
DTSTAMP:20021028T011706Z
SUMMARY:Coffee with Jason
UID:EC9439B1-FF65-11D6-9973-003065F99D04
DTEND;TZID=US/Pacific:20021028T150000
BEGIN:VALARM
TRIGGER;VALUE=DURATION:-P1D
ACTION:DISPLAY
DESCRIPTION:Event reminder\, with comma\\nand line feed
END:VALARM
END:VEVENT
BEGIN:VTIMEZONE
TZID:TEST
LAST-MODIFIED:19870101T000000Z
BEGIN:STANDARD
DTSTART:19971026T020000
RDATE:19971026T020000
TZOFFSETFROM:-0400
TZOFFSETTO:-0500
TZNAME:EST
END:STANDARD
BEGIN:DAYLIGHT
DTSTART:19971026T020000
RDATE:19970406T020000
TZOFFSETFROM:-0500
TZOFFSETTO:-0400
TZNAME:EDT
END:DAYLIGHT
END:VTIMEZONE
END:VCALENDAR"""


__test__ = { "Test readComponents" :
    """
    >>> import StringIO
    >>> from vobject import readComponents
    >>> f2 = StringIO.StringIO(testSilly) 
    >>> silly = readComponents(f2).next()
    >>> silly
    <sillyprofile| [<MORESTUFF{}this line is not folded, but in practice probably ought to be, as it is exceptionally long, and moreover demonstratively stupid>, <SILLYNAME{}name>, <STUFF{}foldedline>]>
    >>> silly.stuff[0]
    <STUFF{}foldedline>
    >>> original = silly.serialize()
    >>> f3 = StringIO.StringIO(original)
    >>> silly2 = readComponents(f3).next()
    >>> silly2.serialize()==original
    True
    >>> s3 = StringIO.StringIO('cn:Babs Jensen\\r\\ncn:Barbara J Jensen\\r\\nsn:Jensen\\r\\nemail:babs@umich.edu\\r\\nphone:+1 313 747-4454\\r\\nx-id:1234567890\\r\\n')
    >>> ex1 = readComponents(s3).next()
    >>> ex1
    <*unnamed*| [<CN{}Babs Jensen>, <CN{}Barbara J Jensen>, <EMAIL{}babs@umich.edu>, <PHONE{}+1 313 747-4454>, <SN{}Jensen>, <X-ID{}1234567890>]>
    >>> ex1.serialize()
    u'CN:Babs Jensen\\r\\nCN:Barbara J Jensen\\r\\nEMAIL:babs@umich.edu\\r\\nPHONE:+1 313 747-4454\\r\\nSN:Jensen\\r\\nX-ID:1234567890\\r\\n'
    """,
    
    "Validate sampledata.txt" :
    """
    >>> import vobject
    >>> f=open("sampledata.txt")
    >>> c=list(vobject.readComponents(f, validate=True))
    >>> c[1].vevent[0].dtend[0].value
    datetime.datetime(1998, 3, 12, 9, 30, tzinfo=<tzicalvtz 'US-Eastern'>)
    >>> c[1].vevent[0].dtend[0].value.astimezone(registry.getTzid("UTC"))
    datetime.datetime(1998, 3, 12, 13, 30, tzinfo=tzutc())
    """,
    
    "Import icaltest" :
    """
    >>> import vobject, StringIO, __init__
    >>> f=StringIO.StringIO(icaltest)
    >>> registry.registerTzid("US/Pacific", registry.getTzid("UTC"))
    >>> c = vobject.readComponents(f, validate=True).next()
    >>> c.vevent[0].valarm[0].trigger[0]
    <TRIGGER{}-1 day, 0:00:00>
    >>> c.vevent[0].rruleset[0]
    datetime.datetime(2002, 10, 28, 14, 0, tzinfo=tzutc())
    >>> c.vevent[0].dtend[0].value
    datetime.datetime(2002, 10, 28, 15, 0, tzinfo=tzutc())
    >>> c.vevent[0].dtstamp[0].value
    datetime.datetime(2002, 10, 28, 1, 17, 6, tzinfo=tzutc())
    >>> c.vevent[0].valarm[0].description[0].value
    u'Event reminder, with comma\\nand line feed'
    >>> c.vevent[0].valarm[0].description[0].serialize()
    u'DESCRIPTION:Event reminder\\\\, with comma\\\\nand line feed\\r\\n'
    >>> f.seek(0) #read icaltest in a different way
    >>> ev =__init__.readICal(f).next()
    >>> ev.rruleset[0] == c.vevent[0].rruleset[0]
    True
    >>> f.seek(0) #read icaltest's top content lines
    >>> top = __init__.readICal(f, topLevel = True).next()
    >>> top.calscale[0]
    <CALSCALE{}GREGORIAN>
    """,
    
    "Parsing tests" :
    """
    >>> import parsing
    >>> parsing.stringToTextValues('')
    ['']
    >>> parsing.stringToTextValues('abcd,efgh')
    ['abcd', 'efgh']
    """,
    
    "Read just TZ" :
    """
    >>> import __init__, StringIO, registry
    >>> __init__.readTZ(StringIO.StringIO(icaltest))
    >>> registry.getTzid('TEST')
    <tzicalvtz 'TEST'>
    """
    }